

#include <SDL.h>
#include <circuit.h>
#include <stdio.h>
#include <vector>
#include <vcg/space/point3.h>

	SDL_Surface *s;

namespace vcg{
typedef Point3<unsigned char> Point3b;
}

int width,height;
vcg::Point3f T(vcg::Point3f p){
	p[0]=p[0] / width*6000 -3000;
	p[2]=p[2] / height*6000 -3000;
	return p;
}

float Tw(float t){
	return t/width*6000-3000;
}
float Th(float t){
	return t/height*6000-3000;
}

struct StreetPx{ static const bool Is(vcg::Point3b c){return ((c==vcg::Point3b(255,0,0))||(c==vcg::Point3b(255,0,255)));}};
struct BuildingPx{static const bool Is(vcg::Point3b c){return (c[1]==255);} };
struct BuildingCornerPx{ static const bool Is(vcg::Point3b c){return (c==vcg::Point3b(255,255,0));} };
struct TreePx{ static const bool Is(vcg::Point3b c){return (c==vcg::Point3b(0,0,255));} };
struct TunnelPx{ static const bool Is(vcg::Point3b c){return (c==vcg::Point3b(255,0,255));} };


vcg::Point3b  & Pix(const int   i, const int   j){
	return *(vcg::Point3b*)& ((unsigned char*)s->pixels)[3*(s->w*j+i)];
}
template <class TYPE>
bool Is(const int   i, const int   j) {
	vcg::Point3b c = Pix(i,j);
	return (TYPE::Is(c));
}

// find one pixel of the given type
template <class TYPE>
vcg::Point2i FindFirst(const int &starti,const int &startj){
	for(int j = startj ; j < s->h;++j)
		for(int i = 0 ; i < s->w;++i){
			if(Is<TYPE>(i,j)) return vcg::Point2i(i,j);
		}
		return vcg::Point2i(-1,-1);
}
template <class TYPE>
std::vector<vcg::Point2i> FindAll(){
	std::vector<vcg::Point2i> res;
	for(int j = 0 ; j < s->h;++j)
		for(int i = 0 ; i < s->w;++i){
			if(Is<TYPE>(i,j,s)) 
				res.push_back(vcg::Point2i(i,j));
		}
		return res;
}

template <class VT>
VT & V(const int i, const int j,const int & w, VT*off){ return off[w*j+i];}

template <class T>
std::vector<vcg::Point2i> Trace(  int  i,   int  j){
	std::vector<vcg::Point2i> res;
	bool * v = (bool *) malloc(s->w*s->h);
	memset(v,0,s->w*s->h);
	int starti = i;
	int startj = j;
	do{
		V(i,j,s->w,v) = true;
		res.push_back(vcg::Point2i(i,j));

		if( Is<T>(i+1,j  ) && !V<bool>(i+1,j  ,s->w,v)) { i = i+1;       } else
		if( Is<T>(i+1,j+1) && !V<bool>(i+1,j+1,s->w,v)) { i = i+1; j=j+1;} else
		if( Is<T>(i  ,j+1) && !V<bool>(i  ,j+1,s->w,v)) {          j=j+1;} else
		if( Is<T>(i-1,j+1) && !V<bool>(i-1,j+1,s->w,v)) { i = i-1; j=j+1;} else
		if( Is<T>(i-1,j  ) && !V<bool>(i-1,j  ,s->w,v)) { i = i-1;       } else
		if( Is<T>(i-1,j-1) && !V<bool>(i-1,j-1,s->w,v)) { i = i-1; j=j-1;} else
		if( Is<T>(i  ,j-1) && !V<bool>(i  ,j-1,s->w,v)) { j = j-1;       } else
		if( Is<T>(i+1,j-1) && !V<bool>(i+1,j-1,s->w,v)) { i = i+1; j=j-1;} else
			break;
	}while(true);
	if ( (abs(i-starti)>1) || (abs(j-startj)>1)){
		printf("error at %d %d\n",i,j);
		res.clear();}

	return res;
}

void SmoothLine( std::vector<vcg::Point2f> & l){
 std::vector<vcg::Point2f> nl = l;
 for(int i = 0; i < nl.size(); ++i)
	 l[i] = (nl[(i+1)%nl.size()]+nl[(i+nl.size()-1)%nl.size()])*0.5;
}

Street MakeStreet(const std::vector<vcg::Point2i> & pxs, float hwidth){
	Street street;
	bool inTunnel = false;
	std::vector<vcg::Point2f> l;
	std::vector<vcg::Point2i>::const_iterator i;
	for(i = pxs.begin(); i != pxs.end(); ++i)
		l.push_back(vcg::Point2f(Tw((*i)[0]),Th((*i)[1])));

	SmoothLine(l);
	SmoothLine(l);
	SmoothLine(l);

	if(l.size() < 3) return Street();
	vcg::Point2f p0 = l[l.size()-1],p1 = l[0],p2 = l[1];
		for(int i = 1; i <  l.size(); ++i){
			vcg::Point2f e0 = p1-p0;
			vcg::Point2f e1 = p2-p1;
			vcg::Point2f bsc = vcg::Point2f(-e0[1],e0[0]) + vcg::Point2f(-e1[1],e1[0]);
			bsc.Normalize();
			vcg::Point2f left2  = p1-hwidth*bsc;
			vcg::Point2f right2 = p1+hwidth*bsc;
			vcg::Point3f left(left2[0],0.0,left2[1]);
			vcg::Point3f right(right2[0],0.0,right2[1]);

			street.path.push_back(vcg::Point2<Vertex>(Vertex(left),Vertex(right)));

			// tunnel
			if( Is<TunnelPx>(pxs[i][0],pxs[i][1])) {
				if(!inTunnel) {street.tunnels.push_back(Street::Tunnel());inTunnel=true;}
				Street::Tunnel & tn = street.tunnels.back();
				tn.path.push_back(vcg::Point2<Vertex>(Vertex(left),Vertex(right)));
				tn.heigth  = 5.f;
			}else inTunnel = false;
					
			p0=p1;
			p1=p2;
			p2=l[(i+1)%l.size()];
		}

		 street.path.push_back( *street.path.begin());


		return street;
}

Circuit::Building MakeBuilding(  int  i,   int  j){
	Circuit::Building res;
	std::vector<vcg::Point2i>::iterator ii;
	std::vector<vcg::Point2i> l =  Trace<BuildingPx>(i,j);

	for(ii = l.begin() ; ii != l.end(); ++ii){
		if(Is<BuildingCornerPx>((*ii)[0],(*ii)[1]))
			res.footprint.push_back(Vertex(vcg::Point3f(Tw((*ii)[0]),0.0,Th((*ii)[1]))));
		Pix((*ii)[0],(*ii)[1])=vcg::Point3b(0,0,0);
	}
	res.height = 10.f;
	return res;
}




int main(int argc, char ** argv){
	Circuit circuit;

	s = SDL_LoadBMP(argv[1]);
	for(int i=0; i < s->w*s->h;++i)
		std::swap(((char*)(s->pixels))[i*3],((char*)(s->pixels))[i*3+2]);

	width  = s->w;	
	height = s->h;	

	// STREET
	vcg::Point2i first = FindFirst<StreetPx>(0,0);
	if(first[0] == -1)
		printf("No 'way', arh arh ahr \n");
	else{
			std::vector<vcg::Point2i> res=Trace<StreetPx>(first[0],first[1]);
			if(!res.empty()){
				circuit.street = MakeStreet(res,2.0);
				circuit.street.entry_points.push_back((circuit.street.path[0][0].P()+circuit.street.path[0][1].P())*0.5); 
			}
	}
	circuit.bbox.Add(vcg::Point3f(-3000,0.0,-3000));
	circuit.bbox.Add(vcg::Point3f( 3000,0.0, 3000));

	// BUILDINGS
	first = FindFirst<BuildingPx>(0,0);
	while(first[0]!=-1){
		circuit.buildings.push_back(Circuit::Building());
		circuit.buildings.back() = MakeBuilding(first[0],first[1]);

		first = FindFirst<BuildingPx>(0,0);
	}

	first = FindFirst<TreePx>(0,0);
	while(first[0]!=-1){
		Pix(first[0],first[1])=vcg::Point3b(0,0,0);
		circuit.trees.push_back(Circuit::Tree());
		circuit.trees.back().pos = vcg::Point3f(Tw(first[0]),0.0,Th(first[1]));
		circuit.trees.back().height = 3.0;
		first = FindFirst<TreePx>(first[0],first[1]);
	}

	circuit.bbox.Add(vcg::Point3f(0.0,10.0,0.0));
	circuit.Save(argv[2]);

	


	return 0;
}