/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/****************************************************************************
  History

$Log: game_server.cpp,v $
Revision 1.11  2007/11/20 13:16:51  ganovelli
missing return type of Comp

Revision 1.10  2007/07/17 10:34:24  ganovelli
added local ip

Revision 1.8  2007/07/16 09:43:13  ganovelli
exchange of screenshot data-draft of drawing others screenshots

Revision 1.7  2007/04/09 23:08:42  ganovelli
added standalone mode

Revision 1.6  2007/03/29 21:52:45  ganovelli
added builder of circuits.

Revision 1.5  2007/03/27 23:10:30  ganovelli
added holes and tunnels to the Street
added buildings to the Circuit
added server to client transmission of each player state
added BRAKING and BOOSTING to the player state
added sun position

Revision 1.4  2007/03/21 10:34:22  ganovelli
added better steering,

Revision 1.3  2007/03/17 12:26:14  ganovelli
* pragma replaced by #ifndef directive
* Player operator= added
* bug in race_simulation.cpp corrected
* msvc2005 solutions added

Revision 1.2  2007/03/11 18:11:59  ganovelli
minor modif


****************************************************************************/
#include<string>
#include "process_message.h"
#include "race_simulation.h"
#include "game_server.h"
#include "simple_circuit_creator.h"
#include "messages.h"
#include "packaging.h"
#include "actions.h"
#include <map>
#include <algorithm>
#include <SDL.h>
#include <SDL_thread.h>


NvmcServer srv;
Race race;

const int in_commands_port = 8084; // udp port to receive commands
const int out_state_port = 8082;	 // udp port to send the state
const int maxtoken = 262144; // 256k
const int in_port = 8086; // tcp port to receive snapshots
const int out_port = 8088;// tcp port to send the snapshots


int NvmcServer::Initialize(char * circuitFile){
	
	/* add a simple circuit to the list of circuits */
	circuits.push_back(Circuit());			
	
	if(circuitFile==NULL)  CreateCircularCircuit(circuits[0]); else circuits[0].Load(circuitFile);
	/* set the circuit as the circuit of the race */
	race.circuit = &circuits[0];
	/* set the sun position */
	circuits[0].sun_position = vcg::Point4f(10000,10000,10000,1.0);
	return 0;
};


class CommandsReceiverThread: public pt::thread{
public:
	CommandsReceiverThread():thread(true){}

 void execute()
 {

	pt::ipmsgserver msvr;
	int h = msvr.bindall(in_commands_port);

	bool quit = false;
    do
    {	
        try
        {
		        pt::string msg = pt::lowercase(msvr.receive(maxtoken));
						pt::string host = pt::iptostring(msvr.get_ip());
						ProcessMessageServer(msg,host);
        }
        catch(pt::estream* e)
        {
						pt::perr.putf("Server error: %s\n", pt::pconst(e->get_message()));
            delete e;
        }
        
    } while (!quit);

	
}
};

pt::ipaddress clients_address;
class StateSenderThread: public pt::thread{
public:
	StateSenderThread():pt::thread(true){trig = new pt::trigger(true,false);}

	pt::trigger * trig;

	void execute()
	{
		int cl=0,part = 0;
		clients_address = pt::ipbcast;

		do{
		++cl;
		trig->wait(); 

		if(!race.players.empty()){// rimpiazzare con un semaforo

			/* First rough draft: cycles on all the players, build a message and send it to all the players */
			std::string msg_str;
			msg_str = Pack(gTime);
			msg_str += Pack(race.circuit->sun_position);
			msg_str +=std::string(" ~ ");
			Race::PlayersIterator pi;

			race.mutex_on_players.enter();
			Race::PlayerContainer players = race.players;
			race.mutex_on_players.leave();

			for(pi = players.begin(); pi != players.end(); ++pi){
				Player pl = (*pi);
				msg_str+=pl.ip_address+std::string(" ");
				pl.mutex_on_data.enter();
				msg_str+=Pack(pl.car.frame);
				msg_str+=Pack(pl.car.vel);
				msg_str+=Pack(pl.car.steering_angle);
				msg_str+=Pack(pl.state);
				msg_str+=std::string("~");
				pl.mutex_on_data.leave();
			}
			/* here msg_str contains the entire state of all the players */
			//ipmessage msg(ipaddress(127, 0, 0, 1), out_port);
			//pt::ipmessage msg(pt::ipaddress(146,48,84,175), out_port);
			pt::ipmessage msg(clients_address, out_state_port);

			if(clock()-part > 1000){
				printf("SEND fps %d\n",cl);
				part = clock();
				cl=0;
			}
			//ipmessage msg(phostbyname((*race.players.begin()).first),out_port);
			try
			{

 					msg.send(pt::string(msg_str.c_str()));
//		printf("message sent\n");

			}
			catch(pt::estream* e)
			{
				//clients_address = pt::ipaddress(127,0,0,1);
				pt::perr.putf("Server error: %s\n", pt::pconst(e->get_message()));
				printf("send_ERROR\n");
					delete e;
			}
		}
	}while(true);
	   
	
}
};

StateSenderThread s;
CommandsReceiverThread r;
void SaveFloat(char * nf,int x,int y,void * pix){
	FILE* f= fopen(nf,"wb");
	fwrite(pix,x*y*3,1,f);
	fclose(f);
}


struct Comp{
	 const bool & operator ()(const Player & p1,const Player & p2)const { return p1.last_updated < p2.last_updated;}  
};

std::list<Player> allPlayers;
pt::mutex mutex_on_allPlayers;

/// thread for one-to-one tcp communication with one specific player
struct DataExchangeClientThread: public pt::thread{

	DataExchangeClientThread():pt::thread(false){}
	DataExchangeClientThread(pt::ipstream *c):pt::thread(false),client(c){}

	struct PlayerTime: public std::pair<Player,int>{
		PlayerTime(const Player &p,const int & t){first=p;second=t;}
	};
	// for each player the time the client "client" was updated with their snapshot
	std::map<Player,int,Comp> players_time;

	pt::ipstream * client;					// the client
	std::string		client_address;		// address of the client

	void execute(){
		try{
		printf("Thread started\n");
		C2SMessage msg;
		pt::string data;
		int readed = 0;
		while(true){
						
			if(client->waitfor(1000)){//check if a new snapshot has been sent
				readed += client->read( &msg ,sizeof(C2SMessage)); // read the body length 
				msg.body = malloc(msg.body_length);
				readed += client->read(msg.body,msg.body_length);
				ProcessMessageServer(msg,pt::iptostring(client->get_ip()));
				printf("New screnshot from %s\n",pt::iptostring(client->get_ip()));
			}

			race.mutex_on_players.enter();
			std::list<Player> players = race.players;
			race.mutex_on_players.leave();

			std::list<Player>::iterator pi;
			std::map<Player,int,Comp>::iterator mpi;
			for(pi = players.begin(); pi != players.end(); ++pi)
				if( (*pi).ip_address != client_address)
			{
				mpi = players_time.find(*pi);
				if(mpi==players_time.end())
					players_time.insert(PlayerTime(*pi,-1));
				else 
					if((*mpi).second  <  (*pi).last_updated){
						(*mpi).second = (*pi).last_updated;// update the time of last sent
						
						// send the screenshot to the client
						S2CMessage srvmess;
						srvmess.header = DATA_PLAYER_SNAPSHOT;
						srvmess.body_length = (*pi).ip_address.length() +1+ 256*256*3;
						unsigned char l1 = (unsigned char )(*pi).ip_address.length();
						client->write(&srvmess,sizeof(S2CMessage));// send the message
						client->write(&l1,1);// lenght of the ipaddress string
						client->write(&(*pi).ip_address[0],(*pi).ip_address.length());
						client->write((*pi).screenshot.pixels,256*256*3);
						client->flush();
						printf("New screnshot from %s sent to %s\n",(*pi).ip_address.c_str(),client_address.c_str());

					}
			}

		}
		}
		catch(pt::estream* e)
			{
					pt::perr.putf("Server error: %s\n", pt::pconst(e->get_message()));
					delete e;
			}

		
	}

};

//std::list<DataExchangeClientThread*>  allClients;

/// thread for one-to-one tcp communication with the players
class DataExchangeThread: public pt::thread{
public:
	DataExchangeThread():pt::thread(false){}

	void execute(){
		while(true){
		try{
				pt::ipstmserver datasrv;			// create the server
				datasrv.bindall(in_port);			// bind to port in_port

				pt::ipstream * client = new pt::ipstream();					// create client

				datasrv.serve(*client);				// for the incoming connection
			
				DataExchangeClientThread * newone =  new DataExchangeClientThread(client);
				newone->client_address = std::string(pt::iptostring(client->get_ip()));
				newone->start();

				//std::list<DataExchangeClientThread*>::iterator i;
				//for(i = allClients.begin(); i!=allClients.end();++i){
				//}
			}
		catch(pt::estream* e)
			{
					pt::perr.putf("Server error: %s\n", pt::pconst(e->get_message()));
					delete e;
			}
		}
	}
};

class Waiter:public pt::thread{ public :Waiter():pt::thread(true){} void execute(){relax(10);}};

int run_race(void * ){
	gTime = clock();
	int part=gTime,cl=0;
	Actions copyof_actionsToDo;
	ActionsIterator ai;
		/* first draft */
	while(true){
		++cl;
		Waiter w;
		w.execute();

		mutex_on_actionsToDo.enter();
		copyof_actionsToDo = actionsToDo;
		actionsToDo.clear();
		mutex_on_actionsToDo.leave();

		for(ai = copyof_actionsToDo.begin(); ai != copyof_actionsToDo.end(); ++ai)
			ProcessAction(*ai);

		Race::PlayersIterator pi;

		race.mutex_on_players.enter();
		Race::PlayerContainer players = race.players;
		race.mutex_on_players.leave();

		for(pi = players.begin(); pi != players.end(); ++pi) 
			Update((clock()-gTime),*pi);
		 
		race.mutex_on_players.enter();
		race.players = players;
		race.mutex_on_players.leave();


		if(clock()-part>1000){
			printf("SIMU fps %d \n",cl, cl);
			cl=0;
			part = clock();
		}

		copyof_actionsToDo.clear();
		gTime = clock();
		s.trig->signal();
	}
}

int NvmcServer::Start(){

	/* start the thread handling command reception (UDP)*/
	r.start();

	/* start the thread handling data reception (TCP)*/
	DataExchangeThread dt;
	dt.start();

	/* start the thread handling ipmessages sending */
	s.start();



	/* game main loop */ 
	run_race((void*)0); 
	
	return 0;
};

int NvmcServer::Terminate(){
	return 0;};
